#include <math.h>
#include <recGbl.h>
#include <devSup.h>
#include <alarm.h>
#include <dbAccess.h>
#include <waveformRecord.h>
#include <epicsVersion.h>
#ifndef BASE_VERSION
/* 3.14 */
#include <epicsExport.h>
#endif

long devDerivateWaveformInit(waveformRecord *record);
long devDerivateWaveformRead(waveformRecord *record);

struct {
    long      number;
    DEVSUPFUN report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN write;
} devDerivateWaveform = {
    5,
    NULL,
    NULL,
    devDerivateWaveformInit,
    NULL,
    devDerivateWaveformRead
};

#ifndef BASE_VERSION
/* 3.14 */
epicsExportAddress(dset, devDerivateWaveform);
#endif

long devDerivateWaveformInit(waveformRecord *record)
{
    switch (record->inp.type) {
    case (CONSTANT) :
        record->nord = 0;
        break;
    case (PV_LINK) :
    case (DB_LINK) :
    case (CA_LINK) :
        break;
    default :
        recGblRecordError(S_db_badField, record,
            "devDerivateWaveform (init_record) Illegal INP field");
        return(S_db_badField);
    }
    return 0;
}

#define DERIVATE(type) \
    type val, delta, old;\
    old = ((type*)(record->bptr))[0];\
    for (i=1; i < nRequest; i++)\
    {\
        val = ((type*)(record->bptr))[i];\
        delta = val - old;\
        old = val;\
        ((type*)(record->bptr))[i] = delta;\
    }\
    for (; i < record->nelm; i++)\
    {\
        ((type*)(record->bptr))[i] = 0;\
    }

long devDerivateWaveformRead(waveformRecord *record)
{
    long status, nRequest, i;

    nRequest=record->nelm;
    /* Get copy of original waveform */
    status = dbGetLink(&record->inp, record->ftvl, record->bptr,
        0, &nRequest);
    if (status)
    {
        recGblSetSevr(record, READ_ALARM, INVALID_ALARM);
        return status;
    }
    record->nord = nRequest;
    switch (record->ftvl)
    {
        case DBF_DOUBLE:
        {
            DERIVATE(double)
            break;
        }
        case DBF_FLOAT:
        {
            DERIVATE(float)
            break;
        }
        case DBF_LONG:
        {
            DERIVATE(long)
            break;
        }
        case DBF_ULONG:
        {
            DERIVATE(unsigned long)
            break;
        }
        case DBF_SHORT:
        {
            DERIVATE(short)
            break;
        }
        case DBF_USHORT:
        case DBF_ENUM:
        {
            DERIVATE(unsigned short)
            break;
        }
        case DBF_CHAR:
        {
            DERIVATE(signed char)
            break;
        }
        case DBF_UCHAR:
        {
            DERIVATE(unsigned char)
            break;
        }
    }
    return 0;
}
