#include <math.h>
#include <recGbl.h>
#include <dbAccess.h>
#include <devSup.h>
#include <waveformRecord.h>
#include <epicsVersion.h>
#ifndef BASE_VERSION
/* 3.14 */
#include <epicsExport.h>
#endif

long devGaussWaveformRead(waveformRecord *record);

struct {
    long      number;
    DEVSUPFUN report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN write;
} devGaussWaveform = {
    5,
    NULL,
    NULL,
    devGaussWaveformRead,
    NULL,
    devGaussWaveformRead
};

#ifndef BASE_VERSION
/* 3.14 */
epicsExportAddress(dset, devGaussWaveform);
#endif

#define INV_SQRT_2PI 0.398942280402

#ifndef __GNUC__
#define __inline static
#endif

__inline double gauss(double x)
{
    return exp(-0.5*x*x);
}

long devGaussWaveformRead(waveformRecord *record)
{
    int i;
    double min = record->lopr;
    double max = record->hopr;
    double range = max - min;
    double sigma;
    long mu;
    
    switch (record->inp.type)
    {
        case CONSTANT:
	    recGblInitConstantLink(&record->inp, DBF_DOUBLE, &sigma);
            break;
        case PV_LINK:
        case DB_LINK:
        case CA_LINK:
            dbGetLink(&record->inp, DBR_DOUBLE, &sigma, NULL, NULL);
    }
    mu = record->rarm;
    switch (record->ftvl)
    {
        case DBF_DOUBLE:
            if (min == 0.0 && record->hopr == 0.0) range = INV_SQRT_2PI/sigma;
            for (i=0; i<record->nelm; i++)
                ((double*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma);
            break;
        case DBF_FLOAT:
            if (min == 0.0 && record->hopr == 0.0) range = INV_SQRT_2PI/sigma;
            for (i=0; i<record->nelm; i++)
                ((float*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma);
            break;
        case DBF_LONG:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 2147483648.0;
            for (i=0; i<record->nelm; i++)
                ((long*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma) + .5;
            break;
        case DBF_ULONG:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 4294967296.0;
            for (i=0; i<record->nelm; i++)
                ((long*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma) + .5;
            break;
        case DBF_SHORT:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 32768.0;
            for (i=0; i<record->nelm; i++)
                ((short*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma) + .5;
            break;
        case DBF_USHORT:
        case DBF_ENUM:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 65536.0;
            for (i=0; i<record->nelm; i++)
                ((short*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma) + .5;
            break;
        case DBF_CHAR:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 128.0;
            for (i=0; i<record->nelm; i++)
                ((char*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma) + .5;
            break;
        case DBF_UCHAR:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 256.0;
            for (i=0; i<record->nelm; i++)
                ((char*)(record->bptr))[i] = min + range * gauss((i-mu)/sigma) + .5;
            break;
    }
    record->nord = record->nelm;
    record->udf = 0;
    return 0;
}
