#include <math.h>
#include <recGbl.h>
#include <devSup.h>
#include <alarm.h>
#include <dbAccess.h>
#include <waveformRecord.h>
#include <epicsVersion.h>
#ifndef BASE_VERSION
/* 3.14 */
#include <epicsExport.h>
#endif

long devIntegrateWaveformInit(waveformRecord *record);
long devIntegrateWaveformRead(waveformRecord *record);

struct {
    long      number;
    DEVSUPFUN report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN write;
} devIntegrateWaveform = {
    5,
    NULL,
    NULL,
    devIntegrateWaveformInit,
    NULL,
    devIntegrateWaveformRead
};

#ifndef BASE_VERSION
/* 3.14 */
epicsExportAddress(dset, devIntegrateWaveform);
#endif

long devIntegrateWaveformInit(waveformRecord *record)
{
    switch (record->inp.type) {
    case (CONSTANT) :
        record->nord = 0;
        break;
    case (PV_LINK) :
    case (DB_LINK) :
    case (CA_LINK) :
        break;
    default :
        recGblRecordError(S_db_badField, record,
            "devIntegrateWaveform (init_record) Illegal INP field");
        return(S_db_badField);
    }
    return 0;
}

#define INTEGRATE(type) \
    type sum=0;\
    for (i=0; i < nRequest; i++)\
    {\
        sum += ((type*)(record->bptr))[i];\
        ((type*)(record->bptr))[i] = sum;\
    }\
    for (; i < record->nelm; i++)\
    {\
        ((type*)(record->bptr))[i] = sum;\
    }

long devIntegrateWaveformRead(waveformRecord *record)
{
    long status, nRequest, i;

    nRequest=record->nelm;
    /* Get copy of original waveform */
    status = dbGetLink(&record->inp, record->ftvl, record->bptr,
        0, &nRequest);
    if (status)
    {
        recGblSetSevr(record, READ_ALARM, INVALID_ALARM);
        return status;
    }
    record->nord = nRequest;
    switch (record->ftvl)
    {
        case DBF_DOUBLE:
        {
            INTEGRATE(double)
            break;
        }
        case DBF_FLOAT:
        {
            INTEGRATE(float)
            break;
        }
        case DBF_LONG:
        {
            INTEGRATE(long)
            break;
        }
        case DBF_ULONG:
        {
            INTEGRATE(unsigned long)
            break;
        }
        case DBF_SHORT:
        {
            INTEGRATE(short)
            break;
        }
        case DBF_USHORT:
        case DBF_ENUM:
        {
            INTEGRATE(unsigned short)
            break;
        }
        case DBF_CHAR:
        {
            INTEGRATE(signed char)
            break;
        }
        case DBF_UCHAR:
        {
            INTEGRATE(unsigned char)
            break;
        }
    }
    return 0;
}
