#include <stdlib.h>
#include <recGbl.h>
#include <dbAccess.h>
#include <devSup.h>
#include <waveformRecord.h>
#include <epicsVersion.h>
#ifndef BASE_VERSION
/* 3.14 */
#include <epicsExport.h>
#endif

long devRandomWaveformRead(waveformRecord *record);

struct {
    long      number;
    DEVSUPFUN report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN write;
} devRandomWaveform = {
    5,
    NULL,
    NULL,
    devRandomWaveformRead,
    NULL,
    devRandomWaveformRead
};

#ifndef BASE_VERSION
/* 3.14 */
epicsExportAddress(dset, devRandomWaveform);
#endif

long devRandomWaveformRead(waveformRecord *record)
{
    int i;
    double min = record->lopr;
    double max = record->hopr;
    double range = max - min;
    
    switch (record->ftvl)
    {
        case DBF_DOUBLE:
            if (min == 0.0 && record->hopr == 0.0) range = 1.0;
            for (i=0; i<record->nelm; i++)
                ((double*)(record->bptr))[i] = min + range * rand() / (RAND_MAX);
            break;
        case DBF_FLOAT:
            if (min == 0.0 && record->hopr == 0.0) range = 1.0;
            for (i=0; i<record->nelm; i++)
                ((float*)(record->bptr))[i] = min + range * rand() / (RAND_MAX);
            break;
        case DBF_LONG:
        case DBF_ULONG:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 4294967296.0;
            for (i=0; i<record->nelm; i++)
                ((long*)(record->bptr))[i] = min + range * ((rand() << 1 & 0xffff0000) + (rand() >> 15)) / (0xffffffff+1.0);
            break;
        case DBF_SHORT:
        case DBF_USHORT:
        case DBF_ENUM:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 65536.0;
            for (i=0; i<record->nelm; i++)
                ((short*)(record->bptr))[i] = min + range * rand() / (RAND_MAX+1.0);
            break;
        case DBF_CHAR:
        case DBF_UCHAR:
            range += 1.0;
            if (min == 0.0 && max == 0.0) range = 256.0;
            for (i=0; i<record->nelm; i++)
                ((char*)(record->bptr))[i] = min + range * rand() / (RAND_MAX+1.0);
            break;
    }
    record->nord = record->nelm;
    record->udf = 0;
    return 0;
}
