#include <recGbl.h>
#include <dbAccess.h>
#include <devSup.h>
#include <waveformRecord.h>
#include <epicsVersion.h>
#ifndef BASE_VERSION
/* 3.14 */
#include <epicsExport.h>
#endif

long devSequenceWaveformRead(waveformRecord *record);

struct {
    long      number;
    DEVSUPFUN report;
    DEVSUPFUN init;
    DEVSUPFUN init_record;
    DEVSUPFUN get_ioint_info;
    DEVSUPFUN write;
} devSequenceWaveform = {
    5,
    NULL,
    NULL,
    devSequenceWaveformRead,
    NULL,
    devSequenceWaveformRead
};

#ifndef BASE_VERSION
/* 3.14 */
epicsExportAddress(dset, devSequenceWaveform);
#endif

long devSequenceWaveformRead(waveformRecord *record)
{
    int i;
    double start;
    short step;
    
    start = 0;
    switch (record->inp.type)
    {
        case CONSTANT:
	    recGblInitConstantLink(&record->inp, DBF_DOUBLE, &start);
            break;
        case PV_LINK:
        case DB_LINK:
        case CA_LINK:
            dbGetLink(&record->inp, DBR_DOUBLE, &start, NULL, NULL);
    }
    step = record->rarm;
    if (!step) step = 1;
    switch (record->ftvl)
    {
        case DBF_DOUBLE:
            if (record->lopr == 0.0 && record->hopr == 0.0)
            {
                for (i=0; i<record->nelm; i++)
                    ((double*)(record->bptr))[i] = start+i*step;
            }
            else
            {
                double fstep = (record->hopr-record->lopr)/(record->nelm-1);
                start = record->lopr;
                for (i=0; i<record->nelm; i++)
                    ((double*)(record->bptr))[i] = start+i*fstep;
            }
            break;
        case DBF_FLOAT:
            if (record->lopr == 0.0 && record->hopr == 0.0)
            {
                for (i=0; i<record->nelm; i++)
                    ((float*)(record->bptr))[i] = start+i*step;
            }
            else
            {
                double fstep = (record->hopr-record->lopr)/(record->nelm-1);
                start = record->lopr;
                for (i=0; i<record->nelm; i++)
                    ((float*)(record->bptr))[i] = start+i*fstep;
            }
            break;
        case DBF_LONG:
        case DBF_ULONG:
            for (i=0; i<record->nelm; i++)
                ((long*)(record->bptr))[i] = (long)(start)+i*step;
            break;
        case DBF_SHORT:
        case DBF_USHORT:
        case DBF_ENUM:
            for (i=0; i<record->nelm; i++)
                ((short*)(record->bptr))[i] = (short)(start)+i*step;
            break;
        case DBF_CHAR:
        case DBF_UCHAR:
            for (i=0; i<record->nelm; i++)
                ((char*)(record->bptr))[i] = (char)(start)+i*step;
            break;
    }
    record->nord = record->nelm;
    record->udf = 0;
    return 0;
}
